/* This program simulates HTTP requests (well as recognized
   by inetrd anyway). Sending SIGINT will cause it to
   print statistics and exit */

#include <stdio.h>
#include <stdlib.h>
#include <syslog.h>
#include <string.h>
#include <fcntl.h>
#include <errno.h>

#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <unistd.h>
#include <time.h>
#include <signal.h>

unsigned long numRequests=0;
time_t start;

void stats(int sig) {
    unsigned long diff=time(0)-start;
    printf("%lu requests in %lu seconds", numRequests, diff);
    printf(" (%lu requests/s)\n", numRequests/diff);
    exit(1);
}

int main(int argc, char* argv[]) {

    char buf[1024];
    struct sockaddr_in servAddr;
    int curr_fd=-1;
    int connect_err;
    int flag_good=0;

    if (argc != 3) {
        printf("Usage: %s ip port\n",  argv[0]);
        return EXIT_FAILURE;
    }

    start = time(0);

    servAddr.sin_family = AF_INET;
    servAddr.sin_port = htons(atoi(argv[2]));
    servAddr.sin_addr.s_addr = inet_addr(argv[1]);

    signal(SIGINT, stats);

    setvbuf(stdout,NULL,_IONBF,0);

    printf(
    "Legend:\n"
    "c = connect returned EAGAIN\n"
    "C = connect returned other error (errno shown)\n"
    "r = server closed before read\n"
    "R = other error on read (errno shown)\n"
    "s = error for socket create (errno shown)\n"
    );

    for(;;) {
        if (curr_fd == -1) {
            curr_fd = socket(AF_INET, SOCK_STREAM, 0);
            if (curr_fd == -1) {
                printf("s[%d]",errno);
                continue;
            }
        }
        connect_err = connect(curr_fd, (struct sockaddr *) &servAddr, sizeof(servAddr));
        if (connect_err != -1) {
            int read_size;
            write(curr_fd, "GET crap\r\nHost: myhost\r\n\r\n", 26); /* simulate a valid HTTP request */
            read_size=read(curr_fd, buf, sizeof(buf));
            if (!read_size)
                putchar('r');
            else if (read_size<0)
                printf("R[%d]",errno);
            else {
                if (flag_good) putchar('.');
                    numRequests++;
            }
            close(curr_fd); curr_fd=-1;
        } else if (errno != EAGAIN) {
            printf("C[%d]",errno);
            close(curr_fd); curr_fd=-1;
        } else {
            putchar('c');
            flag_good=1;
        }
    }
}
